/* filename:    vdirfid2.g
** description: The program computes generates the identification of
**              the VAR impulse response functions
**              functions for the DSGE model
** created:     11/06/01
*/

/********************************************************
** Open files with Parameter Draws
*/

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

/******************************************************************
**         Load IRF Draws 
*/

lpath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";

lvarirf  = lpath $+ "\\" $+ llam $+ lsamp $+ "varirf0"; 
ldsgeirf = lpath $+ "\\" $+ llam $+ lsamp $+ "dsgeirf0"; 

open fhvarirf =  ^lvarirf for read;
open fhdsgeirf = ^ldsgeirf for read;

/* Initialize Output files
*/
opath  = resupath $+ "\\mhrun" $+ mhrun $+"\\";

ovarirf  = opath $+ "\\" $+ llam $+ lsamp $+ "varirf"; 
odsgeirf = opath $+ "\\" $+ llam $+ lsamp $+ "dsgeirf"; 

create fhodsgeirf = ^odsgeirf with IRF, nvar^2*nirf, 8;
create fhovarirf  = ^ovarirf with IRF, nvar^2*nirf, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 1;

do until eofloop; 

   /* Read blocks of size nsim
   */
   mirf0sim = readr( fhdsgeirf, nblock );
   virf0sim = readr( fhvarirf, nblock );
   
   nread   = rows(mirf0sim);

   "Loop      " loopct;
   "Blocksize " nread;
   eofloop = eof(fhvarirf);
   eofloop = 1;

   mirfsim = zeros(nread,nvar^2*nirf);
   virfsim = zeros(nread,nvar^2*nirf);
   
   j = 1;
   do until j > nread;

      if fmod(j,100)==0;".";; pause(1e-20); endif;

      AAirf = reshape(mirf0sim[j,.]',nvar^2,nirf)'; /* DSGE Model Response */
      BBirf = reshape(virf0sim[j,.]',nvar^2,nirf)'; /* VAR Response */
      
      AA0 = reshape(AAirf[1,.],nvar,nvar)';
      {AArot,AAtr} = qqr(AA0');
      
      AAtr  = AAtr';
      AArot = AArot';
      
      BB0 = reshape(BBirf[1,.],nvar,nvar)';
      
      AAtrsign = -(diag(AAtr) .< 0) + (diag(AAtr) .GE 0);
      BB0sign = -(diag(BB0) .< 0) + (diag(BB0) .GE 0);
      
      signswitch = eye(rows(BB0)) .* (AAtrsign .* BB0sign) ;
      
      AAtr = AAtr*signswitch;
      AArot = signswitch*AArot;
            
      AAirftr = AAirf;
      BBirftr = BBirf*(AArot.*.eye(nvar));
      
      /* output responses are cumulative
      */
      AAirftr[.,1 4 7] = cumsumc(AAirftr[.,1 4 7]);
      BBirftr[.,1 4 7] = cumsumc(BBirftr[.,1 4 7]);
      
      mirfsim[j,.] = vec(AAirftr)';
      virfsim[j,.] = vec(BBirftr)';
      
      j = j+1;
  endo;
      
  /* Write Output
  */
  wr = writer(fhodsgeirf,mirfsim);
  wr = writer(fhovarirf,virfsim);

  loopct = loopct + 1;

endo;

closeall fhodsgeirf, fhovarirf, fhdsgeirf, fhvarirf;
