/* filename:    bmarghess.g
** description: posterior mode batch
** created:     11/05/01
*/

new;


/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
__output = 1;       /** 1 for verbose of CSMIN**/
__graph  = 1;       /** 1 for graph mode **/
_verbose = 2;       /** 0 for silence, 1 for compact, 2 for detailed**/
_fcmptol = 1E-10;

/** only the following 2 lines are model dependent **/
#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;
#include var_spec.g;

#include pathspec.g;

/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

npara    = rows(prior);
pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;


/** start log file **/
outstr = resupath $+ "\\hess.log";
output file = ^outstr reset;

/******************************************************** 
** Iterate over lambda
*/

Tind = 1;
do until Tind > 1;

   /********************************************************
   * Import data series
   ********************************************************/
   #include loaddata.g;
   #include var_loaddata.g; 

   lamind = 1;
   do until lamind > rows(lamgrid);

      lambda = lamgrid[lamind];
      gosub runproc;

      lamind = lamind+1;

   endo;

   Tind = Tind+1;  

endo;
end;

runproc:

"";
"";
"";
"=========================================================";
"Index L,T" lamind Tind;
"LAMBDA = " lambda;
"---------------------------------------------------------";

#include p_marghess.g

return;


#include p_varprior.g;
